/*************************************************************

(C) ISEP 2004
(C) Professores de ADAV 2004

O ISEP, na qualidade de autor do presente software, garante 
o direito de utilizao, alterao, evoluo e distribuio 
do mesmo sem qualquer tipo de restries, excepto:
	- esta mensagem no pode ser retirada
	- os nomes das classes no podem ser alterados

*************************************************************/

using System;
using System.Data;
using System.Data.OleDb;

namespace ShopClassLibrary
{
	public interface IProduct
	{
		DataSet GetById(string user, string pass, int productId);
		DataSet FindByName(string user, string pass, string namePattern);
		int GetStock(string user, string pass, int productId, out ShopStatus status);
		ShopStatus InStock(string user, string pass, int productId, int quant);
		ShopStatus AddStock(string user, string pass, int productId, int quant);
	}

	/// <summary>
	/// Summary description for Product.
	/// </summary>
	internal class Product : IProduct
	{
		#region IProduct
		/// <summary>
		/// pesquisa um determinado produto por cdigo de produto
		/// </summary>
		/// <param name="User">username de login na aplicao</param>
		/// <param name="Pass">password de login na aplicao</param>
		/// <returns>a informao do produto</returns>
		public DataSet GetById(string user, string pass, int productId)
		{
			DataSet rs = null;

			// <--- CODIGO EM FALTA --->

			return rs;
		}

		/// <summary>
		/// pesquisa produtos por nome
		/// </summary>
		/// <param name="User">username de login na aplicao</param>
		/// <param name="Pass">password de login na aplicao</param>
		/// <returns>informao dos produtos que obedeam ao critrio de busca</returns>
		public DataSet FindByName(string user, string pass, string namePattern)
		{
			DataSet rs = null;

			// <--- CODIGO EM FALTA --->

			return rs;
		}

		/// <summary>
		/// devolve a quantidade em stock de um determinado produto
		/// </summary>
		/// <param name="User">username de login na aplicao</param>
		/// <param name="Pass">password de login na aplicao</param>
		/// <param name="ProductID">cdigo de produto a pesquizar</param>
		/// <param name="status"></param>
		/// <returns></returns>
		public int GetStock(string user, string pass, int productId, out ShopStatus status)
		{
			int qt = 0;
			status = ShopStatus.Ok;

			// <--- CODIGO EM FALTA --->

			return qt;
		}

		/// <summary>
		/// verifica se existe uma determinada quantidade de um produto em armazem 
		/// </summary>
		/// <param name="User">username de login na aplicao</param>
		/// <param name="Pass">password de login na aplicao</param>
		/// <param name="ProductID">cdigo de produto a pesquizar</param>
		/// <param name="Quant">quantidade desejada de produto</param>
		/// <returns>cdigo de estado relativo  operao</returns></returns></returns>
		public ShopStatus InStock(string user, string pass, int productId, int quant)
		{
			ShopStatus status = ShopStatus.Ok;

			return status;
		}

		public ShopStatus AddStock(string user, string pass, int productId, int quant)
		{
			OleDbConnection conn = null;
			ShopStatus status;

			//
			// validar parametros de entrada
			//
			if (quant <= 0)
				return ShopStatus.InvalidArgument;

			try
			{
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();
				
				//
				// validar o utilizador
				//
				status = UtilDB.ValidateUser(conn, null, user, pass);
				if (status != ShopStatus.Ok)
					return status;

				//
				// criar comando SQL a executar
				//
				string sSqlCmd = "Update Products SET StockQtd = StockQtd + ? WHERE ProductID = " + productId.ToString(); 
				OleDbCommand cmd = new OleDbCommand(sSqlCmd, conn);
				cmd.Parameters.Add("qt", quant);

				//
				// executar o comando 
				//
				int updRows = cmd.ExecuteNonQuery();

				//
				// verificar o resultado
				//
				if (updRows == 0)
					// no actualizou registo
					status = ShopStatus.NotOk;
				else
					// actualizou registo
					status = ShopStatus.Ok;
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <AddStock> class 'Product': " + ex.Message);
				status = ShopStatus.Error;
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}

			return status;
		}
		#endregion

		#region mtodos internos

		/// <summary>
		/// verifica se existe uma determinada quantidade de um produto em armazem.
		/// este mtodo no faz parte do interface pblico da classe, apenas  
		/// utilizado internamente na ClassLibrary
		/// </summary>
		/// <param name="ProductID">cdigo de produto a pesquizar</param>
		/// <param name="Quant">quantidade desejada de produto</param>
		/// <returns>cdigo de estado relativo  operao</returns>
		internal ShopStatus InStock(OleDbConnection conn, OleDbTransaction tx, int ProductID, int Quant)
		{
			//
			// efectuar o select
			//
			DataSet rs = UtilDB.GetById(conn, tx, "Products", "ProductID", ProductID);
			if (rs == null)
				return ShopStatus.NotOk;

			//
			// validar stock
			//
			if (rs.Tables["products"].Rows.Count > 0)
				return ((int)(rs.Tables["Products"].Rows[0]["StockQtd"]) >= Quant ? ShopStatus.Ok : ShopStatus.InsufficientStock);
			else
				return ShopStatus.InvalidProductId;
		}

		#endregion
	}
}
